#include <cstdio>
#include <queue>
using namespace std;

typedef int flow_t;
const int MAXN = 1805, MAXM = 5000, DIRECTED = 0, UNDIRECTED = 1;
const flow_t INFTY = 0x3fffffff;
int N, S, T, now;
struct edge {
	flow_t remain;
	int endVertexId, nextEdgeId;
}e[MAXM << 1];
struct vertex {
	int firstEdgeId, level, firstUnsaturEdgeId;
}v[MAXN];

void _addEdge(int begin, int end, flow_t c) {
	e[now].remain = c;
	e[now].endVertexId = end;
	e[now].nextEdgeId = v[begin].firstEdgeId;
	v[begin].firstEdgeId = now++;
}

void addEdge(int begin, int end, flow_t c, int edgeType) {
	_addEdge(begin, end, c);
	_addEdge(end, begin, edgeType * c);
}

void init() {
	now = 0;
	for (int i = 0; i < N; ++i) v[i].firstEdgeId = -1;
}

bool markLevel() {
	for (int i = 0; i < N; ++i)
		v[i].level = -1, v[i].firstUnsaturEdgeId = v[i].firstEdgeId;
	v[S].level = 0;
	queue<int> Q;
	Q.push(S);
	while (!Q.empty()) {
		int x = Q.front();
		Q.pop();
		for (int i = v[x].firstEdgeId; i >= 0; i = e[i].nextEdgeId)
				if (e[i].remain && v[e[i].endVertexId].level < 0)
					v[e[i].endVertexId].level = v[x].level + 1, Q.push(e[i].endVertexId);
	}
	return v[T].level > 0;
}

flow_t extendFlow(int x, flow_t flow) {
	if (x == T) return flow;
	flow_t t, total = 0;
	for (int &i = v[x].firstUnsaturEdgeId; i >= 0; i = e[i].nextEdgeId) {
		if (v[e[i].endVertexId].level == v[x].level + 1 && e[i].remain) {
			if (t = extendFlow(e[i].endVertexId, min(flow, e[i].remain)))
				e[i].remain -= t, e[i ^ 1].remain += t, flow -= t, total += t;
			if (0 == flow) break;
		}
	}
	return total;
}

flow_t Dinic() {
	flow_t flow, total = 0;
	while (markLevel())
		while (flow = extendFlow(S, INFTY))
			total += flow;
	return total;
}

const int MAXL = 35;
int n, m, c;
char a[MAXL][MAXL];
int cost[26];

bool ok(int x, int y) {
	return x >= 0 && x < m && y >= 0 && y < n;
}

int inId(int x, int y) {
	return x * n + y;
}

int outId(int x, int y) {
	return x * n + y + n * m;
}

void buildGraph() {
	N = n * m * 2 + 1;
	T = n * m * 2;
	init();
	const int dx[] = {-1, 0, 1, 0};
	const int dy[] = {0, -1, 0, 1};
	for (int i = 0; i < m; ++i) {
		for (int j = 0; j < n; ++j) {
			addEdge(inId(i, j), outId(i, j), (a[i][j] == '.' || a[i][j] == 'B') ? INFTY : cost[a[i][j] - 'a'], DIRECTED);
			if (a[i][j] == 'B')
				S = inId(i, j);
			for (int k = 0; k < 4; ++k) {
				if (ok(i + dx[k], j + dy[k]))
					addEdge(outId(i + dx[k], j + dy[k]),
									inId(i, j),
									INFTY,
									DIRECTED);
			}
			if (i == 0 || i == m - 1 || j == 0 || j == n - 1)
				addEdge(outId(i, j), T, INFTY, DIRECTED);
		}
	}
}

int main() {
	scanf("%d%d%d", &n, &m, &c);
	for (int i = 0; i < m; ++i)
		scanf(" %s", a[i]);
	for (int i = 0; i < c; ++i)
		scanf("%d", &cost[i]);
	buildGraph();
	int ans = Dinic();
	printf("%d\n", ans >= INFTY ? -1 : ans);
	return 0;
}
